#include <CMS/CMS8S5880.H>
//*****************************************************************************************//
//**********************************VER 1.0 data:20190524**********************************//
//**********************************    author : fred    **********************************//
//********************************* I2C master send&receive********************************//
//************************** send data and then receive data*******************************//
//*****************************************************************************************//

//***********define constant*********************//

#define 	MAIN_CLOCK_FREQ	16000000ul
	
#define			bit_run		0x01				//I2CMCR.0 run   bit 0:disable 1:enable(transmit&receive)
#define			bit_start	0x02				//I2CMCR.1 start bit 0:disable 1:enable
#define			bit_stop	0x04				//I2CMCR.2 stop  bit 0:disable 1:enable
#define			bit_ack		0x08				//I2CMCR.3 ACK   bit 0:disable 1:enable

//*******************variable*********************//

volatile u8 send_CNT=0,rece_CNT=0;
bit	F_start_rec,F_rec;

//*******************initial*********************//

void I2C_MS_init();								//master 
void INT_init();								//INT initinal

//*******************MAIN*********************//

void main()
{
	
	send_CNT	= 0;
	F_start_rec	= 0;
	F_rec		= 0;
	
	I2C_MS_init();
	INT_init();
	
	while(1)
	{
		clrwdt();
	}
	
}

/*************************************************
function name: I2C_MS_init()
function work: initial the I2C master function
input data   : null
output data  : null
note		 : setting the I2C master function
*************************************************/

void I2C_MS_init()
{
	
#define			I2C_400K		400000
#define			I2C_100K		100000
#define			I2C_CLOCK		I2C_100K
	
	FUNCCR		&=	0xEF;						//SCL,SDA for I2C pin
	P04CFG		=	0x03;						//switch P04 to SCL
	P05CFG		=	0x03;						//switch P05 to SDA
	
	P0TRIS		|=	0x30;						//output setting
	P0UP		|=	0x30;						//setting output pull high
	
	P0OD		|=	0x30;						//ver1.1 add opendrain output setting
	
//	
//	FUNCCR		|=	0x10;						//[SCL],[SDA]for I2C pin
//	P21CFG		=	0x03;
//	P35CFG		=	0x03;

//	P2TRIS		|=	0x02;
//	P2UP		|=	0x02;						//setting output pull high
//	P2RD		|=	0x02;						//ver1.1 add opendrain output setting

//	P3TRIS		|=	0x20;
//	P3UP		|=	0x20;						//setting output pull high
//	P3OD		|=	0x20;						//ver1.1 add opendrain output setting

	I2CMBUF		=	0x059;
	
	//the data register for send in write operation
	
	//I2CMTP	=	((MAIN_CLOCK_FREQ/(10*2*I2C_CLOCK))-1);
	I2CMTP	=	7;
	//SCL clock period :2* (1+TIMER_PRD)* 10* Tsys
	
	I2CMSA	=	0xA0+0;
	
	//bit7~1:SA6~SA0 master address for I2C
	//bit0 R/S  0:address right and send data 1:address right and receive data

	
	I2CMCR		=	(bit_run+bit_start);
	

	
}
/*************************************************
function name: INT_init()
function work: initial the INT function
input data   : null
output data  : null
note		 : setting the interrupt function
*************************************************/
void INT_init()						//interrupt initinal
{

	EIE2	=	0x40;
	
//Bit7	SPIIE	SPI enable interrupt bit 0:disable 1:enable
//Bit6	I2CIE	I2C enable interrupt bit 0:disable 1:enable
//Bit5	WDTIE	WDT enable interrupt bit 0:disable 1:enable
//Bit4	ADCIE	ADC enable interrupt bit 0:disable 1:enable
//Bit3	PWMIE	PWM enable interrupt bit 0:disable 1:enable(EIP2.PPWM must be the same as EIP1.PP3PWM)
//Bit1	ET4	 Timer4 enable interrupt bit 0:disable 1:enable
//Bit0	ET3	 Timer3 enable interrupt bit 0:disable 1:enable
  	
	EIP2	=	0x00;
	
//Bit7	PSPI	SPI interrupt priority control bit 0:high priority 1:low priority
//Bit6	PI2C	I2C interrupt priority control bit 0:high priority 1:low priority
//Bit5	PWDT	WDT interrupt priority control bit 0:high priority 1:low priority
//Bit4	PADC	ADC interrupt priority control bit 0:high priority 1:low priority
//Bit3	PPWM	PWM interrupt priority control bit 0:high priority 1:low priority
//Bit1	PT4  Timer4 interrupt priority control bit 0:high priority 1:low priority
//Bit0	PT3  Timer3 interrupt priority control bit 0:high priority 1:low priority


	EA		=	1;		//Bit7	EA	Global interrupt enable bit 0:disable 1:enable
	
}

/*************************************************
function name: I2C_INT()
function work: I2C INT function
input data   : null
output data  : null
note		 : send and receive the data
*************************************************/
void I2C_INT()		interrupt	I2C_VECTOR
{
	if(EIF2&0x40)
	{
		
		if(I2CMCR&0x80)							//master send data
		{
			I2CMCR		= 0x00;					//clear flag
			
			if(send_CNT<3)
			{
				
				I2CMBUF = 0x02+send_CNT;
				I2CMCR	= bit_run;
				send_CNT++;
			}
			else
			{
				if(send_CNT!=128)
				{
				I2CMCR	= bit_stop;
				send_CNT= 128;
				}
				if(F_start_rec==0)
				{
					F_start_rec=1;
					I2CMSA	= 0xA0+1;
					I2CMCR	= (bit_run+bit_start+bit_ack);
				}
				else
				{
					rece_CNT++;
					F_rec	= 1;
					I2CMCR	= (bit_run+bit_ack);
				}
			}
			if(rece_CNT>128 && F_rec==1)
			{
				I2CMCR	= bit_stop;
			}
				
		}

	}
}


