#include <CMS/CMS8S5880.H>
//*****************************************************************************************//
//**********************************VER 1.0 data:20190524**********************************//
//**********************************    author : fred    **********************************//
//*********************************** SPI  send&receive***********************************//
//*****************************************************************************************//
//***SPI master send and receive data, send the data and the next data must insert delay***//
//**************define constant******************//



//*******************variable*********************//

u8 data_WR;
u8 data_RD[256];

//*******************initial*********************//

void MSPI_init();								//master
void INT_init();								//interrupt initial
void SPI_INT();

//******************* main  *********************//
void main()
{
	data_WR	= 0;
	
	MSPI_init();
	INT_init();
	
	SPDR	= data_WR;
	
	while(1)
	{
		clrwdt();
	}
}

/*************************************************
function name: MSPI_init()
function work: initial the SPI master function
input data   : null
output data  : null
note		 : setting the SPI master function
*************************************************/
void MSPI_init()
{
	P15CFG		=	0x04;					//switch P15 to SCLK
	P16CFG		=	0x04;					//switch P16 to MOSI
	P17CFG		=	0x04;					//switch P17 to MISO
	
	SPCR		=	0x50;

	//Bit6	SPEN	SPI enable bit 0:SPI disable 1:SPI enable
	//Bit5	SPR2	SPI CLOCK bit2
	//Bit4	MSTR	SPI mode select 0:slave 1:master
	//Bit3	CPOL	SPI idle state 0:SCL IDLE LOW 1:SPI IDLE HIGH
	//Bit2	CPHA	SPI CLOCK phase select 0:the first rising/falling edge sample 1:the second rising/falling edge sample
	//Bit1	SPR1
	//Bit0	SPR0 	2^{(SPR2~0)+2} prescaler e.g. 000:FSYS/4 001:FSYS/8 ......
	
	SPSR		|=	0x01;
	//	
	//Bit7	SPISIF	SPI transmit over flag read only 0:transmit isnot over 1: transmit over
	//Bit6	WCOL	SPI write conflict interrupt flag bit, read only;
	//				1= SPDR operation conflict when SPI transmission is not completed (read SPSR first, then read/write SPDR and then reset);
	//				0= no write conflicts
	//Bit4	MODF	SPI mode error interrupt flag bit, read only;
	//1=SPI is configured as master control mode, inside of the chip from the SSn signal (independent of the NSS port in host mode) is pulled to low level 
	//0=SPI schema is error free
	//Bit1	SSICS	SPI slave mode must be 1
	//Bit0	SSCEN	SPI master mode NSSx (x=3, 5-7) output control bit.
	//				1= automatic selection of NSSx output, which can be controlled by SSCR to select output of NSS3 and nss5-nss7.(4 wires mode)
	//				0= automatic NSSx output selection disabled, NSS3, nss5-nss7 all output 1.(3 wires mode)
	
	SSCR		=	0x0F7;		//work in 4pin mode
	P14CFG 		= 	0x04;		//P14 config NSS3
	//SPI selects control bits from the device, I =3, 5-7 (other bits don't matter).
	//				0=NSSi allocation as output from the device selection.
	//				1=NSSi forced output 1
	
//	SSCR		=	0x07F;
//	P36CFG 		= 	0x04;		//P36 config NSS7
//	SSCR		=	0x0DF;
//	P21CFG 		= 	0x04;		//P21 config NSS5	
//	SSCR		=	0x0BF;
//	P22CFG 		= 	0x04;		//P22 config NSS6	
	
	
}

/*************************************************
function name: INT_init()
function work: initial the INT function
input data   : null
output data  : null
note		 : setting the interrupt function
*************************************************/
void INT_init()						//intterrupt initinal
{
	IE		=	0x00;
	
//Bit7	EA	Global interrupt enable bit 0:disable 1:enable
//Bit6	ES1	UART1 interrupt enable bit  0:disable 1:enable
//Bit5	ET2	Timer2 global interrupt enable bit  0:disable 1:enable
//Bit4	ES0	UART0 interrupt enable bit  0:disable 1:enable
//Bit3	ET1	Timer1 interrupt enable bit  0:disable 1:enable
//Bit2	EX1	INT1 interrupt enable bit  0:disable 1:enable
//Bit1	ET0	Timer0 interrupt enable bit  0:disable 1:enable
//Bit0	EX0	INT0 interrupt enable bit  0:disable 1:enable

	EIE2	=	0x80;
	
//Bit7	SPIIE	SPI enable interrupt bit 0:disable 1:enable
//Bit6	I2CIE	I2C enable interrupt bit 0:disable 1:enable
//Bit5	WDTIE	WDT enable interrupt bit 0:disable 1:enable
//Bit4	ADCIE	ADC enable interrupt bit 0:disable 1:enable
//Bit3	PWMIE	PWM enable interrupt bit 0:disable 1:enable(EIP2.PPWM must be the same as EIP1.PP3PWM)
//Bit1	ET4	 Timer4 enable interrupt bit 0:disable 1:enable
//Bit0	ET3	 Timer3 enable interrupt bit 0:disable 1:enable

	EIP2	=	0x00;
	
//Bit7	PSPI	SPI interrupt priority control bit 0:high priority 1:low priority
//Bit6	PI2C	I2C interrupt priority control bit 0:high priority 1:low priority
//Bit5	PWDT	WDT interrupt priority control bit 0:high priority 1:low priority
//Bit4	PADC	ADC interrupt priority control bit 0:high priority 1:low priority
//Bit3	PPWM	PWM interrupt priority control bit 0:high priority 1:low priority
//Bit1	PT4  Timer4 interrupt priority control bit 0:high priority 1:low priority
//Bit0	PT3  Timer3 interrupt priority control bit 0:high priority 1:low priority


	EA		=	1;		//Bit7	EA	Global interrupt enable bit 0:disable 1:enable
	
}

/*************************************************
function name: SPI_INT()
function work: SPI_INT function
input data   : null
output data  : null
note		 : send and receive the data
*************************************************/
void SPI_INT()		interrupt	SPI_VECTOR
{					
								//if want to send data ,write data to SPDR;
								//if want to receive data ,read SPDR data;
	if(EIF2&0x80)
	{
		if(SPSR&0x80)			//SPI transmit over flag read only 0:transmit isnot over 1: transmit over
		{
			data_RD[data_WR]=SPDR;
			
			_nop6_();			//receive and send need delay time
			_nop6_();
			_nop6_();
			_nop6_();
			_nop6_();
			_nop6_();
			_nop6_();
			
			data_WR++;
			SPDR	=	data_WR;
			
		}
		if(SPSR&0x40)			//SPI write conflict interrupt flag bit, read only;
		{
			SPDR	=	0xAA;
		}		
		if(SPSR&0x10)			//SPI mode error interrupt flag bit, read only;
		{
			SPDR	=	0xA5;
		}
	}
			
}